@@ -74,10 +74,10 @@ sub PDL_BOOT {
 #define aTHX_
 #endif
    if (SvTRUE (ERRSV)) Perl_croak(aTHX_ "%s",SvPV_nolen (ERRSV));
-   CoreSV = perl_get_sv("PDL::SHARE",FALSE);  /* SV* value */
-   if (CoreSV==NULL)
+   if (!(CoreSV = perl_get_sv("PDL::SHARE",FALSE))) /* SV* value */
      Perl_croak(aTHX_ "We require the PDL::Core module, which was not found");
-   $symname = INT2PTR(Core*,SvIV( CoreSV ));  /* Core* value */
+   if (!($symname = INT2PTR(Core*,SvIV( CoreSV )))) /* Core* value */
+     Perl_croak(aTHX_ "Got NULL pointer for $symname");
    if ($symname->Version != PDL_CORE_VERSION)
      Perl_croak(aTHX_ "[$symname->Version: \%d PDL_CORE_VERSION: \%d XS_VERSION: \%s] $module needs to be recompiled against the newly installed PDL", $symname->Version, PDL_CORE_VERSION, XS_VERSION);
 
@@ -206,6 +206,7 @@ qq|
 
 $pref.pm : $pmdep
 	$perlrun \"$pp_call_arg\" $src
+	\$(TOUCH) $pref.pm
 
 @generanda : $pref.pm
 	\$(NOECHO) \$(NOOP)
@@ -292,7 +293,7 @@ sub pdlpp_mkgen {
 
   my @pairs = ();
   my $manifest = ExtUtils::Manifest::maniread($file);
-  for (keys %$manifest) {
+  for (sort keys %$manifest) {
     next if $_ !~ m/\.pd$/;     # skip non-pd files
     next if $_ =~ m/^(t|xt)\//; # skip *.pd files in test subdirs
     next unless -f $_;
@@ -521,7 +522,7 @@ my %flags = (
     vaffine => { FLAG => "OPT_VAFFTRANSOK" },
     anychgd => { FLAG => "ANYCHANGED" },
     dimschgd => { FLAG => "PARENTDIMSCHANGED" },
-    tracedebug => { FLAG => "TRACEDEBUG", set => 1},
+    tracedebug => { set => 1},
 );
 
 sub generate_core_flags {
@@ -530,46 +531,19 @@ sub generate_core_flags {
     # to ndarray's state
     foreach my $name ( sort keys %flags ) {
         my $flag = "PDL_" . ($flags{$name}{FLAG} || uc($name));
-        if ( $flags{$name}{set} ) {
-            print <<"!WITH!SUBS!";
+        my $with_mode = $flags{$name}{set} || $flags{$name}{postset};
+        printf <<'EOF', $name, ($with_mode ? ",mode=0" : ''), ($with_mode ? "        int mode\n" : '');
 int
-$name(x,mode=0)
+%s(x%s)
         pdl *x
-        int mode
-        CODE:
-        if (items>1)
-           { setflag(x->state,$flag,mode); }
-        RETVAL = ((x->state & $flag) > 0);
-        OUTPUT:
-        RETVAL
-
-!WITH!SUBS!
-        } elsif ($flags{$name}{postset}) {
-            print <<"!WITH!SUBS!";
-int
-$name(x,mode=0)
-        pdl *x
-        int mode
-        CODE:
-        RETVAL = ((x->state & $flag) > 0);
-        if (items>1)
-           { setflag(x->state,$flag,mode); }
-        OUTPUT:
-        RETVAL
-
-!WITH!SUBS!
-        } else {
-            print <<"!WITH!SUBS!";
-int
-$name(self)
-        pdl *self
-        CODE:
-        RETVAL = ((self->state & $flag) > 0);
-        OUTPUT:
-        RETVAL
-
-!WITH!SUBS!
-        }
+%s        CODE:
+EOF
+        my $set = "        if (items>1) setflag(x->state,$flag,mode);\n";
+        my $ret = "        RETVAL = ((x->state & $flag) > 0);\n";
+        print $set if $flags{$name}{set};
+        print $ret;
+        print $set if $flags{$name}{postset};
+        print "        OUTPUT:\n        RETVAL\n\n";
     } # foreach: keys %flags
 }
 
