@@ -54,7 +54,7 @@
 #   Implicit conditions are NewXSSymTab and Name
 #
 # The MakeComp rule creates the compiled representation accessed by $COMP()
-#  PDL::PP::Rule::MakeComp->new("MakeCompiledRepr", ["MakeComp","CompNames","CompObjs"],
+#  PDL::PP::Rule::MakeComp->new("MakeCompiledRepr", ["MakeComp","CompObj"],
 #		      "COMP")
 # PDL::PP::Rule::MakeComp->new($target,$conditions,$symbol)
 #   $target and $symbol must be scalars.
@@ -139,21 +139,9 @@ sub any_targets_exist {
 sub all_conditions_exist {
     my $self = shift;
     my $pars = shift;
-
-    my $conditions = $self->{conditions};
-
-    foreach my $condition (@$conditions) {
-
-	# skip if not a required condition
-	next if substr($condition,0,1) eq "_";
-
-	unless (exists $pars->{$condition}) {
-	    $self->report("--skipping since CONDITION $condition does not exist\n");
-	    return 0;
-	}
-    }
-
-    return 1;
+    return 1 unless my @nonexist = grep !ref() && !exists $pars->{$_}, @{$self->{conditions}};
+    $self->report("--skipping since CONDITIONs (@nonexist) do not exist\n");
+    0;
 }
 
 # $rule->should_apply($pars);
@@ -173,10 +161,7 @@ sub should_apply {
 # my @args = $self->extract_args($pars);
 sub extract_args {
     my ($self, $pars) = @_;
-    @$pars{ map {
-        (my $condition = $_) =~ s/^_//; # initial _ = optional condition
-        $condition;
-    } @{ $self->{conditions} } };
+    @$pars{ map ref($_) eq "SCALAR" ? $$_ : $_, @{ $self->{conditions} } };
 }
 
 # Apply the rule using the supplied $pars hash reference.
@@ -405,33 +390,54 @@ our @CARP_NOT;
 
 our @ISA = qw (PDL::PP::Rule);
 
+sub badflag_isset {
+  PDL::PP::pp_line_numbers(__LINE__-1, "($_[0]->state & PDL_BADVAL)")
+}
+
 # Probably want this directly in the apply routine but leave as is for now
 #
 sub dosubst_private {
-    my ($src,$sname,$pname,$name) = @_;
+    my ($src,$sname,$pname,$name,$sig) = @_;
     my $ret = (ref $src ? $src->[0] : $src);
     my %syms = (
-		((ref $src) ? %{$src->[1]} : ()),
-		PRIV => sub {return "$sname->$_[0]"},
-		COMP => sub {return "$pname->$_[0]"},
-		CROAK => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL->pdl_barf(\"Error in $name:\" $_[0])")},
-		NAME => sub {return $name},
-		MODULE => sub {return $::PDLMOD},
-		SETPDLSTATEBAD  => sub { PDL::PP::pp_line_numbers(__LINE__-1, "$_[0]\->state |= PDL_BADVAL") },
-		SETPDLSTATEGOOD => sub { PDL::PP::pp_line_numbers(__LINE__-1, "$_[0]\->state &= ~PDL_BADVAL") },
-		ISPDLSTATEBAD   => sub { PDL::PP::pp_line_numbers(__LINE__-1, "(($_[0]\->state & PDL_BADVAL) > 0)") },
-		ISPDLSTATEGOOD  => sub { PDL::PP::pp_line_numbers(__LINE__-1, "(($_[0]\->state & PDL_BADVAL) == 0)") },
-		BADFLAGCACHE    => sub { PDL::PP::pp_line_numbers(__LINE__-1, "badflag_cache") },
-	       );
-    while(
-	  $ret =~ s/\$(\w+)\(([^()]*)\)/
-	  (defined $syms{$1} or
-	   confess("$1 not defined in '$ret'!")) and
-	  (&{$syms{$1}}($2))/ge
-	 ) {};
+      ((ref $src) ? %{$src->[1]} : ()),
+      PRIV => sub {return "$sname->$_[0]"},
+      COMP => sub {return "$pname->$_[0]"},
+      CROAK => sub {PDL::PP::pp_line_numbers(__LINE__-1, "return PDL->make_error(PDL_EUSERERROR, \"Error in $name:\" $_[0])")},
+      NAME => sub {return $name},
+      MODULE => sub {return $::PDLMOD},
+      SETPDLSTATEBAD  => sub { PDL::PP::pp_line_numbers(__LINE__-1, "$_[0]\->state |= PDL_BADVAL") },
+      SETPDLSTATEGOOD => sub { PDL::PP::pp_line_numbers(__LINE__-1, "$_[0]\->state &= ~PDL_BADVAL") },
+      ISPDLSTATEBAD   => \&badflag_isset,
+      ISPDLSTATEGOOD  => sub {"!".badflag_isset($_[0])},
+      BADFLAGCACHE    => sub { PDL::PP::pp_line_numbers(__LINE__-1, "badflag_cache") },
+      PDLSTATESETBAD => sub { PDL::PP::pp_line_numbers(__LINE__-1, $sig->objs->{$_[0]}->do_pdlaccess."->state |= PDL_BADVAL") },
+      PDLSTATESETGOOD => sub { PDL::PP::pp_line_numbers(__LINE__-1, $sig->objs->{$_[0]}->do_pdlaccess."->state &= ~PDL_BADVAL") },
+      PDLSTATEISBAD => sub {badflag_isset($sig->objs->{$_[0]}->do_pdlaccess)},
+      PDLSTATEISGOOD => sub {"!".badflag_isset($sig->objs->{$_[0]}->do_pdlaccess)},
+      PP => sub { $sig->objs->{$_[0]}->do_physpointeraccess },
+      P => sub { (my $o = $sig->objs->{$_[0]})->{FlagPhys} = 1; $o->do_pointeraccess; },
+      PDL => sub { $sig->objs->{$_[0]}->do_pdlaccess },
+      SIZE => sub { $sig->ind_obj($_[0])->get_size },
+   );
+    while (my ($before, $kw, $args, $other) = macro_extract($ret)) {
+      confess("$kw not defined in '$ret'!") if !$syms{$kw};
+      $ret = join '', $before, $syms{$kw}->($args), $other;
+    }
     $ret;
 }
 
+sub macro_extract {
+  require Text::Balanced;
+  my ($text) = @_;
+  return unless $text =~ /\$(\w+)\s*(?=\()/;
+  my ($before, $kw, $other) = ($`, $1, $');
+  (my $bracketed, $other) = Text::Balanced::extract_bracketed($other, '(")');
+  $bracketed = substr $bracketed, 1, -1; # chop off brackets
+  $bracketed =~ s:^\s*(.*?)\s*$:$1:;
+  ($before, $kw, $bracketed, $other);
+}
+
 sub new {
     my $class = shift;
 
@@ -444,7 +450,7 @@ sub new {
     die "\$target must be a scalar for PDL::PP::Rule::Substitute" if ref $target;
     die "\$condition must be a scalar for PDL::PP::Rule::Substitute" if ref $condition;
 
-    $class->SUPER::new($target, [$condition, "StructName", "ParamStructName", "Name"],
+    $class->SUPER::new($target, [$condition, qw(StructName ParamStructName Name SignatureObj)],
 				  \&dosubst_private);
 }
 
@@ -475,8 +481,6 @@ our @ISA = qw (PDL::PP::Rule::Substitute);
 my @std_childparent = (
 	CHILD => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[1]->'.(join ',',@_).")")},
 	PARENT => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[0]->'.(join ',',@_).")")},
-	CHILD_P => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[1]->'.(join ',',@_).")")},
-	PARENT_P => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[0]->'.(join ',',@_).")")},
 	CHILD_PTR => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[1])')},
 	PARENT_PTR => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[0])')},
 );
@@ -502,9 +506,6 @@ sub extract_args {
     return ([$code,{@std_childparent}],$sname,$name);
 }
 
-#  PDL::PP::Rule->new("MakeCompiledRepr", ["MakeComp","CompNames","CompObjs"],
-#		      sub {subst_makecomp("COMP",@_)}),
-#
 # PDL::PP::Rule::MakeComp->new($target,$conditions,$symbol)
 #   $target and $symbol must be scalars.
 #
@@ -520,26 +521,25 @@ our @ISA = qw (PDL::PP::Rule);
 # This is a copy of the main one for now. Need a better solution.
 #
 my @std_redodims = (
-		    SETNDIMS => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL->reallocdims(__it,$_[0])")},
-		    SETDIMS => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL->setdims_careful(__it)")},
-		    SETDELTATHREADIDS => sub {PDL::PP::pp_line_numbers(__LINE__-1, '
-		{int __ind; PDL->reallocthreadids($CHILD_PTR(),
-			$PARENT(nthreadids));
-		for(__ind=0; __ind<$PARENT(nthreadids); __ind++) {
-			$CHILD(threadids[__ind]) =
-				$PARENT(threadids[__ind]) + ('.$_[0].');
-		}
-		}
-		')});
+  SETNDIMS => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL_RETERROR(PDL_err, PDL->reallocdims(__it,$_[0]));")},
+  SETDIMS => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL_RETERROR(PDL_err, PDL->setdims_careful(__it));")},
+  SETDELTATHREADIDS => sub {PDL::PP::pp_line_numbers(__LINE__, <<EOF)},
+{int __ind; PDL_RETERROR(PDL_err, PDL->reallocthreadids(\$CHILD_PTR(), \$PARENT(nthreadids)));
+for(__ind=0; __ind<\$PARENT(nthreadids); __ind++)
+  \$CHILD(threadids[__ind]) = \$PARENT(threadids[__ind]) + ($_[0]);
+}
+EOF
+);
 
 # Probably want this directly in the apply routine but leave as is for now
 #
 sub subst_makecomp_private {
-	my($which,$mc,$cn,$co) = @_;
+	my($which,$mc,$cobj) = @_;
+	my ($cn,$co) = !$cobj ? () : map $cobj->$_, qw(othernames otherobjs);
 	return [$mc,{
 		PDL::PP::Rule::Substitute::Usual::get_std_childparent(),
 		($cn ?
-			(('DO'.$which.'DIMS') => sub {PDL::PP::pp_line_numbers(__LINE__, join '',
+			(('DO'.$which.'ALLOC') => sub {join('',
 				map $$co{$_}->get_malloc("\$$which($_)"),
 				    grep $$co{$_}->need_malloc, @$cn)}) :
 			()
@@ -595,7 +595,8 @@ our $macros_xs = <<'EOF';
                             by pp_bless ? (CS) */ \
   HV *bless_stash = 0; \
   SV *parent = 0; \
-  int   nreturn;
+  int   nreturn = 0; \
+  (void)nreturn;
 
 #define PDL_XS_PACKAGEGET \
   PDL_COMMENT("Check if you can get a package name for this input value.  ") \
@@ -613,6 +614,7 @@ our $macros_xs = <<'EOF';
   if (strcmp(objname,"PDL") == 0) { PDL_COMMENT("shortcut if just PDL") \
      name ## _SV = sv_newmortal(); \
      name = PDL->null(); \
+     if (!name) PDL->pdl_barf("Error making null pdl"); \
      PDL->SetSV_PDL(name ## _SV, name); \
      if (bless_stash) name ## _SV = sv_bless(name ## _SV, bless_stash); \
   } else { \
@@ -666,9 +668,10 @@ PDL_COMMENT("   /* Memory access */                                         ")
 #define PDL_XS_INPLACE_CHECK(in) \
     if (in->state & PDL_INPLACE) barf("inplace input but output given");
 
-#define PDL_FREE_CODE(trans, comp_free_code, ntpriv_free_code) \
-    PDL_TR_CLRMAGIC(trans); \
-    comp_free_code \
+#define PDL_FREE_CODE(trans, destroy, comp_free_code, ntpriv_free_code) \
+    if (destroy) { \
+	comp_free_code \
+    } \
     if ((trans)->dims_redone) { \
 	ntpriv_free_code \
     }
@@ -881,7 +884,6 @@ sub pp_addxs {
 #   FirstKey => ...,
 #   Code => pp_line_numbers(__LINE__, $x . $y . $c),
 #   OtherKey => ...
-
 sub pp_line_numbers ($$) {
 	my ($line, $string) = @_;
 	# The line needs to be incremented by one for the bookkeeping to work
@@ -890,8 +892,7 @@ sub pp_line_numbers ($$) {
 	my (undef, $filename) = caller;
 	# Escape backslashes:
 	$filename =~ s/\\/\\\\/g;
-	my @to_return = "\n#line $line \"$filename\"\n";
-
+	my @to_return = "\nPDL_LINENO_START $line \"$filename\"\n";
 	# Look for threadloops and loops and add # line directives
 	foreach (split (/\n/, $string)) {
 		# Always add the current line.
@@ -899,22 +900,42 @@ sub pp_line_numbers ($$) {
 		# If we need to add a # line directive, do so after incrementing
 		$line++;
 		if (/%\{/ or /%}/) {
-			push @to_return, "#line $line \"$filename\"\n";
+			push @to_return, "PDL_LINENO_END\n";
+			push @to_return, "PDL_LINENO_START $line \"$filename\"\n";
 		}
 	}
-
+	push @to_return, "PDL_LINENO_END\n";
 	return join('', @to_return);
 }
+sub _pp_linenumber_fill {
+  my ($file, $text) = @_;
+  my (@stack, @to_return) = [$file, 1];
+  foreach (split (/\n/, $text)) {
+    $_->[1]++ for @stack;
+    push(@to_return, $_), next if !/^(\s*)PDL_LINENO_(?:START (\S+) "(.*)"|(END))$/;
+    my ($ci, $new_line, $new_file, $is_end) = ($1, $2, $3, $4);
+    if ($is_end) {
+      pop @stack;
+      push @to_return, qq{$ci#line $stack[-1][1] "$stack[-1][0]"};
+    } else {
+      push @stack, [$new_file, $new_line-1];
+      push @to_return, qq{$ci#line @{[$stack[-1][1]+1]} "$stack[-1][0]"};
+    }
+  }
+  join '', map "$_\n", @to_return;
+}
 
 sub _file_same {
   my ($from_text, $to_file) = @_;
   require File::Map;
   File::Map::map_file(my $to_map, $to_file, '<');
-  $from_text eq $to_map;
+  s/^[^\n]*#line[^\n]*?\n//gm for $from_text, (my $to_text = $to_map);
+  $from_text eq $to_text;
 }
 sub _write_file {
   my ($file, $text) = @_;
-  return if -f $file && length($text) == -s $file && _file_same($text, $file);
+  $text = _pp_linenumber_fill($file, $text);
+  return if -f $file && _file_same($text, $file);
   open my $fh, '>', $file or confess "open $file: $!";
   binmode $fh; # to guarantee length will be same for same contents
   print $fh $text;
@@ -948,14 +969,12 @@ sub pp_done {
             $pdl_boot, $::PDLXSBOOT, $PP::boundscheck,
           );
         _write_file("$::PDLPREF.xs", $text);
-
-unless (nopm) {
+        return if nopm;
 	$::PDLPMISA = "'".join("','",@::PDLPMISA)."'";
 	$::PDLBEGIN = "BEGIN {\n$::PDLBEGIN\n}"
 		unless $::PDLBEGIN =~ /^\s*$/;
-	open my $fh, ">", "$::PDLPREF.pm" or die "Couldn't open pm file: $!\n";
         $::FUNCSPOD = $::DOCUMENTED ? "\n\n=head1 FUNCTIONS\n\n=cut\n\n" : '';
-	print $fh <<EOF;
+        _write_file("$::PDLPREF.pm", join "\n\n", <<EOF, $::PDLBEGIN, $::PDLPM{Top}, $::FUNCSPOD, @::PDLPM{qw(Middle Bot)}, '# Exit with OK status', "1;\n");
 #
 # GENERATED WITH PDL::PP! Don't modify!
 #
@@ -974,22 +993,7 @@ $::PDL_IFBEGINWRAP[0]
    push \@PDL::Core::PP, __PACKAGE__;
    bootstrap $::PDLMOD $::PDLMODVERSION;
 $::PDL_IFBEGINWRAP[-1]
-
-$::PDLBEGIN
-
-$::PDLPM{Top}
-
-$::FUNCSPOD
-
-$::PDLPM{Middle};
-
-$::PDLPM{Bot}
-
-# Exit with OK status
-
-1;
 EOF
-      }  # unless (nopm)
 } # end pp_done
 
 sub _pp_parsename {
@@ -1113,9 +1117,6 @@ EOF
 
 }
 
-# Worst memleaks: not freeing things at redodims or
-# final free time (thread, dimmed things).
-
 use Carp;
 $SIG{__DIE__} = \&Carp::confess if $::PP_VERBOSE;
 
@@ -1271,82 +1272,50 @@ sub typemap {
   # Do checks...
   #
   # First reconstruct the type declaration to look up in type_kind
-  my $full_type=TidyType($type->get_decl('')); # Skip the variable name
+  my $full_type=TidyType($type->get_decl('', {VarArrays2Ptrs=>1})); # Skip the variable name
   die "The type =$full_type= does not have a typemap entry!\n" unless exists($type_kind{$full_type});
   my $typemap_kind = $type_kind{$full_type};
   # Look up the conversion from the INPUT typemap. Note that we need to do some
   # massaging of this.
   my $input = $input_expr{$typemap_kind};
-  # Remove all before =:
-  $input =~ s/^(.*?)=\s*//; # This should not be very expensive
-  # Replace $arg with $arg
-  $input =~ s/\$arg/$arg/;
-  # And type with $full_type
-  $input =~ s/\$type/$full_type/;
-
+  $input =~ s/^(.*?)=\s*//s; # Remove all before =
+  $input =~ s/\$(var|\{var\})/$oname/g;
+  $input =~ s/\$(arg|\{arg\})/$arg/g;
+  $input =~ s/\$(type|\{type\})/$full_type/g;
   return ($input);
 }
 
-sub OtherPars_nft {
-    my ($otherpars,$sig) = @_;
-    my $dimobjs = $sig && $sig->dims_obj;
-    my(@names,%types,$type);
-    # support 'int ndim => n;' syntax
-    for (PDL::PP::Signature::nospacesplit(';',$otherpars)) {
-	if (/^\s*([^=]+)\s*=>\s*(\S+)\s*$/) {
-	    my ($ctype,$dim) = ($1,$2);
-	    $ctype =~ s/\s+$//; # get rid of trailing ws
-	    print "OtherPars: setting dim '$dim' from '$ctype'\n" if $::PP_VERBOSE;
-	    $type = PDL::PP::CType->new(undef,$ctype);
-	    croak "can't set unknown dimension"
-		unless defined($dimobjs->{$dim});
-	    $dimobjs->{$dim}->set_from($type);
-	} elsif(/^\s*\(\s*void\s*\)/) {
-	    # suppressing unused param warning - skip
-	    next;
-	} else {
-	    $type = PDL::PP::CType->new(undef,$_);
-	}
-	my $name = $type->protoname;
-	push @names,$name;
-	$types{$name} = $type;
-    }
-    return (\@names,\%types);
-}
-
 sub wrap_vfn {
   my (
     $code,$rout,$func_header,
-    $all_func_header,$sname,$pname,$ptype,
-    $name,
+    $all_func_header,$sname,$pname,$ptype,$extra_args,
   ) = @_;
   my $str = join "\n", grep $_, $all_func_header, $func_header, $code;
+  my $opening = 'pdl_error PDL_err = {0, NULL, 0};';
+  my $closing = 'return PDL_err;';
   PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-void $rout(pdl_trans *$sname) {
+pdl_error $rout(pdl_trans *$sname$extra_args) {
+$opening
 @{[$ptype ? "  $ptype *$pname = $sname->params;" : ""]}
-$str}
+$str$closing}
 EOF
 }
-my @vfn_args_always = qw(_AllFuncHeader StructName ParamStructName ParamStructType);
+my @vfn_args_always = (\"AllFuncHeader", qw(StructName ParamStructName ParamStructType));
 sub make_vfn_args {
-  my ($which) = @_;
+  my ($which, $extra_args) = @_;
   ("${which}Func",
-    ["${which}CodeSubd","${which}FuncName","_${which}FuncHeader",
+    ["${which}CodeSubd","${which}FuncName",\"${which}FuncHeader",
       @vfn_args_always
     ],
-    sub {$_[1] eq 'NULL' ? '' : wrap_vfn(@_,lc $which)}
+    sub {$_[1] eq 'NULL' ? '' : wrap_vfn(@_,$extra_args//'')}
   );
 }
 
-my @xscode_args_always = (
-  "_NewXSCHdrs",
-  "RunFuncCall",
-);
 sub make_xs_code {
-  my($xscode_before,$xscode_after,$hdr,
+  my($xscode_before,$xscode_after,$str,
     $xs_c_headers,
     @bits) = @_;
-  my($str,$boot,$prelude) = PDL::PP::pp_line_numbers(__LINE__-1, $hdr);
+  my($boot,$prelude);
   if($xs_c_headers) {
     $prelude = join '' => ($xs_c_headers->[0], @bits, $xs_c_headers->[1]);
     $boot = $xs_c_headers->[2];
@@ -1381,25 +1350,6 @@ sub callPerlInit {
     indent($ret,$ci);
 } #sub callPerlInit()
 
-sub NT2Decls__ {
-  my($opts,$onames,$otypes) = @_;
-  my $decl;
-  my $dopts = {};
-  $dopts->{VarArrays2Ptrs} = 1 if $opts->{ToPtrs};
-  for(@$onames) {
-      my $d = $otypes->{$_}->get_decl($_,$dopts);
-      $decl .= PDL::PP::pp_line_numbers(__LINE__-1, "$d;") if $d;
-  }
-  $decl;
-}
-
-sub NT2Free_p {
-  my($onames,$otypes,$symbol) = @_;
-  my $decl = join '', map $otypes->{$_}->get_free("\$$symbol($_)",
-    { VarArrays2Ptrs => 1 }), @$onames;
-  $decl ? PDL::PP::pp_line_numbers(__LINE__, $decl) : '';
-}
-
 ###########################################################
 # Name       : extract_signature_from_fulldoc
 # Usage      : $sig = extract_signature_from_fulldoc($fulldoc)
@@ -1474,7 +1424,7 @@ $PDL::PP::deftbl =
    # ie should we bother with bad values for this routine?
    # 1     - yes,
    # 0     - no, maybe issue a warning
-   PDL::PP::Rule->new("BadFlag", "_HandleBad",
+   PDL::PP::Rule->new("BadFlag", \"HandleBad",
 		      "Sets BadFlag based upon HandleBad key",
 		      sub { $_[0] }),
 
@@ -1571,7 +1521,7 @@ $PDL::PP::deftbl =
    # could be really clever and include the sig to see about
    # input/output params, for instance
    
-   PDL::PP::Rule->new("BadDoc", ["BadFlag","Name","_CopyBadStatusCode"],
+   PDL::PP::Rule->new("BadDoc", ["BadFlag","Name",\"CopyBadStatusCode"],
               'Sets the default documentation for handling of bad values',
       sub {
          my ( $bf, $name, $code ) = @_;
@@ -1607,7 +1557,7 @@ $PDL::PP::deftbl =
          return $fulldoc;
       }
    ),
-   PDL::PP::Rule->new("PdlDoc", ["Name","_Pars","OtherPars","Doc","_BadDoc"],
+   PDL::PP::Rule->new("PdlDoc", ["Name",\"Pars","OtherPars","Doc",\"BadDoc"],
       sub {
         my ($name,$pars,$otherpars,$doc,$baddoc) = @_;
         return '' if !defined $doc # Allow explcit non-doc using Doc=>undef
@@ -1694,10 +1644,10 @@ EOD
    PDL::PP::Rule::Returns::Zero->new("IsAffineFlag"),
    PDL::PP::Rule::Returns->new("TwoWayFlag", "TwoWay", "PDL_ITRANS_TWOWAY"),
    PDL::PP::Rule::Returns::Zero->new("TwoWayFlag"),
-   PDL::PP::Rule::Returns->new("DefaultFlowFlag", "DefaultFlow", "PDL_ITRANS_DO_DATAFLOW_F|PDL_ITRANS_DO_DATAFLOW_B"),
+   PDL::PP::Rule::Returns->new("DefaultFlowFlag", "DefaultFlow", "PDL_ITRANS_DO_DATAFLOW_ANY"),
    PDL::PP::Rule::Returns::Zero->new("DefaultFlowFlag"),
 
-   PDL::PP::Rule->new("RedoDims", ["EquivPDimExpr","_EquivDimCheck"],
+   PDL::PP::Rule->new("RedoDims", ["EquivPDimExpr",\"EquivDimCheck"],
       sub {
         my($pdimexpr,$dimcheck) = @_;
         $pdimexpr =~ s/\$CDIM\b/i/g;
@@ -1705,7 +1655,7 @@ EOD
           int i,cor;
           '.$dimcheck.'
           $SETNDIMS($PARENT(ndims));
-          $DOPRIVDIMS();
+          $DOPRIVALLOC();
           $PRIV(offs) = 0;
           for(i=0; i<$CHILD(ndims); i++) {
             cor = '.$pdimexpr.';
@@ -1733,7 +1683,7 @@ EOD
         # parse 'good' code
         $good =~ s/\$EQUIVCPOFFS\(([^()]+),([^()]+)\)/\$PP(CHILD)[$1] = \$PP(PARENT)[$2]/g;
         $good =~ s/\$EQUIVCPTRUNC\(([^()]+),([^()]+),([^()]+)\)/\$PP(CHILD)[$1] = ($3) ? 0 : \$PP(PARENT)[$2]/g;
-        return PDL::PP::pp_line_numbers(__LINE__-1, $good) if !$bflag;
+        return $good if !$bflag;
         # parse 'bad' code
         $bad  =~ s/\$EQUIVCPOFFS\(([^()]+),([^()]+)\)/if( \$PPISBAD(PARENT,[$2]) ) { \$PPSETBAD(CHILD,[$1]); } else { \$PP(CHILD)[$1] = \$PP(PARENT)[$2]; }/g;
         $bad =~ s/\$EQUIVCPTRUNC\(([^()]+),([^()]+),([^()]+)\)/ if( ($3) || \$PPISBAD(PARENT,[$2]) ) { \$PPSETBAD(CHILD,[$1]); } else {\$PP(CHILD)[$1] = \$PP(PARENT)[$2]; }/g;
@@ -1771,14 +1721,14 @@ EOD
       #                    --CED 27-Jan-2003
       #
       # this just reverses PARENT & CHILD in the expansion of
-      # the $EQUIVCPOFFS macro (ie compared to CodefromEquivCPOffsCode)
+      # the $EQUIVCPOFFS macro (ie compared to Code from EquivCPOffsCode)
       sub {
         my ($good, $bflag) = @_;
         my $bad  = $good;
         # parse 'good' code
         $good =~ s/\$EQUIVCPOFFS\(([^()]+),([^()]+)\)/\$PP(PARENT)[$2] = \$PP(CHILD)[$1]/g;
         $good =~ s/\$EQUIVCPTRUNC\(([^()]+),([^()]+),([^()]+)\)/if(!($3)) \$PP(PARENT)[$2] = \$PP(CHILD)[$1] /g;
-        return PDL::PP::pp_line_numbers(__LINE__-1, $good) if !$bflag;
+        return $good if !$bflag;
         # parse 'bad' code
         $bad  =~ s/\$EQUIVCPOFFS\(([^()]+),([^()]+)\)/if( \$PPISBAD(CHILD,[$1]) ) { \$PPSETBAD(PARENT,[$2]); } else { \$PP(PARENT)[$2] = \$PP(CHILD)[$1]; }/g;
         $bad =~ s/\$EQUIVCPTRUNC\(([^()]+),([^()]+),([^()]+)\)/if(!($3)) { if( \$PPISBAD(CHILD,[$1]) ) { \$PPSETBAD(PARENT,[$2]); } else { \$PP(PARENT)[$2] = \$PP(CHILD)[$1]; } } /g;
@@ -1797,7 +1747,8 @@ EOD
 
 # Parameters in the 'a(x,y); [o]b(y)' format, with
 # fixed nos of real, unthreaded-over dims.
-   PDL::PP::Rule->new("SignatureObj", ["Pars","BadFlag"],
+# Also "Other pars", the parameters which are usually not pdls.
+   PDL::PP::Rule->new("SignatureObj", ["Pars","BadFlag","OtherPars"],
       sub { PDL::PP::Signature->new(@_) }),
 
  # Set CallCopy flag for simple functions (2-arg with 0-dim signatures)
@@ -1815,23 +1766,6 @@ EOD
 	  !$sig->objs->{$sig->names->[1]}{FlagTyped};
       }),
 
-# "Other pars", the parameters which are usually not pdls.
-
-   PDL::PP::Rule->new(["OtherParNames","OtherParTypes"], ["OtherPars","SignatureObj"], \&OtherPars_nft),
-
-   PDL::PP::Rule->new("NewXSArgs", ["SignatureObj","OtherParNames","OtherParTypes"],
-      sub {
-        my($sig,$onames,$oobjs) = @_;
-        my $pdltype = PDL::PP::CType->new(undef,"pdl *__foo__");
-        my $nxargs = [
-          ( map {[$_,$pdltype]} @{ $sig->names } ),
-          ( map {[$_,$oobjs->{$_}]} @$onames )
-        ];
-        return $nxargs;
-      }),
-
-   PDL::PP::Rule::Returns->new("PMCode", undef),
-
    PDL::PP::Rule->new(["InplaceCode","InplaceCheck"], ["SignatureObj","Inplace"],
 		      'Insert code (just after HdrCode) to ensure the routine can be done inplace',
       # insert code, after the autogenerated xs argument processing code
@@ -1874,28 +1808,21 @@ EOD
    PDL::PP::Rule::Returns::EmptyString->new("HdrCode", [],
 					    'Code that will be inserted at the end of the autogenerated xs argument processing code VargArgsXSHdr'),
 
-   # globalnew implies internal usage, not XS
-   PDL::PP::Rule::Returns->new("VarArgsXSHdr","GlobalNew",undef),
    PDL::PP::Rule->new("VarArgsXSHdr",
-      ["Name","NewXSArgs","SignatureObj","OtherParTypes",
-       "PMCode","HdrCode","InplaceCode","InplaceCheck","_CallCopy","_Bitwise"],
-      'XS code to process arguments on stack based on supplied Pars argument to pp_def; GlobalNew has implications how/if this is done',
-      # This subroutine is called when no 'otherpars' exist.
-      # This writes an XS header which handles variable argument lists,
-      # thus avoiding the perl layer in calling the routine. D. Hunt 4/11/00
-      #
-      # The use of 'DO NOT SET!!' looks ugly.
+      ["Name","SignatureObj",
+       "HdrCode","InplaceCode","InplaceCheck",\"CallCopy",\"Bitwise"],
+      'XS code to process input arguments based on supplied Pars argument to pp_def; not done if GlobalNew or PMCode supplied',
       sub {
-        my($name,$xsargs,$sig,$optypes,
-           $pmcode,$hdrcode,$inplacecode,$inplacecheck,$callcopy,$bitwise) = @_;
+        my($name,$sig,
+           $hdrcode,$inplacecode,$inplacecheck,$callcopy,$bitwise) = @_;
         # Don't do var args processing if the user has pre-defined pmcode
-        return 'DO NOT SET!!' if ($pmcode);
         my $ci = '  ';  # current indenting
-        my $pars = join "\n",map {$ci.$_->[1]->get_decl($_->[0]).";"} @$xsargs;
-        my @args   = map { $_->[0] } @$xsargs;
+        my $pars = join "\n",map "$ci$_ = 0;", $sig->alldecls(1, 0);
+        my @args = $sig->alldecls(0, 1);
         my %out = map +($_=>1), $sig->names_out_nca;
         my %outca = map +($_=>1), $sig->names_oca;
         my %tmp = map +($_=>1), $sig->names_tmp;
+        my $optypes = $sig->otherobjs(1);
         my %other  = map { $_ => exists($$optypes{$_}) } @args;
         # remember, othervars *are* input vars
         my $nout   = (grep { $_ } values %out);
@@ -1908,11 +1835,10 @@ EOD
         my $ninout = $nin + $nout;
         my $nallout = $nout + $noutca;
         my $usageargs = join (",", @args);
-        $ci = '  ';  # Current indenting
         # Generate declarations for SV * variables corresponding to pdl * output variables.
         # These are used in creating output and temp variables.  One variable (ex: SV * outvar1_SV;)
         # is needed for each output and output create always argument
-        my $svdecls = join "\n", map "${ci}SV *${_}_SV = NULL;", keys %tmp, $sig->names_out;
+        my $svdecls = join "\n", map "${ci}SV *${_}_SV = NULL;", sort(keys %tmp), $sig->names_out;
         my $clause_inputs = ''; my %already_read; my $cnt = 0;
         foreach my $x (@args) {
             last if $out{$x} || $tmp{$x} || $outca{$x} || $other{$x};
@@ -1947,8 +1873,8 @@ EOD
         if ($nmaxonstack == $ninout) {
             $clause2 = '';
         } else {
-            $clause2 = "\n  else if (items == $ninout) { PDL_COMMENT(\"all but temps on stack, read in output, create temps\")" .
-                "    nreturn = $noutca;\n";
+            $clause2 = "\n  else if (items == $ninout) { PDL_COMMENT(\"all but temps on stack, read in output, create temps\")\n" .
+                "${ci}nreturn = $noutca;\n";
             $cnt = 0;
             foreach my $x (@args) {
                 if ($other{$x}) {
@@ -2026,11 +1952,11 @@ END
         PDL::PP::pp_line_numbers(__LINE__-1, "PDL_XS_RETURN($clause1)");
       }),
 
-   PDL::PP::Rule->new("NewXSHdr", ["NewXSName","NewXSArgs"],
+   PDL::PP::Rule->new("NewXSHdr", ["NewXSName","SignatureObj"],
       sub {
-        my($name,$pars) = @_;
-        my $shortpars = join ',',map {$_->[0]} @$pars;
-        my $longpars = join "\n",map {"\t".$_->[1]->get_decl($_->[0])} @$pars;
+        my($name,$sig) = @_;
+        my $shortpars = join ',', $sig->alldecls(0, 1);
+        my $longpars = join "\n", map "\t$_", $sig->alldecls(1, 1);
         return<<END;
 
 void
@@ -2039,28 +1965,25 @@ $longpars
 END
       }),
    PDL::PP::Rule::InsertName->new("RunFuncName", 'pdl_${name}_run'),
-   PDL::PP::Rule->new("NewXSCHdrs", ["RunFuncName","NewXSArgs","GlobalNew"],
+   PDL::PP::Rule->new("NewXSCHdrs", ["RunFuncName","SignatureObj","GlobalNew"],
       sub {
-        my($name,$pars,$gname) = @_;
-        my $longpars = join ",",map {$_->[1]->get_decl($_->[0])} @$pars;
-        return ["void $name($longpars) {","}",
+        my($name,$sig,$gname) = @_;
+        my $longpars = join ",", $sig->alldecls(1, 0);
+        my $opening = 'pdl_error PDL_err = {0, NULL, 0};';
+        my $closing = 'return PDL_err;';
+        return ["pdl_error $name($longpars) {$opening","$closing}",
                 "PDL->$gname = $name;"];
       }),
-   PDL::PP::Rule->new(["RunFuncCall","RunFuncHdr"],["RunFuncName","NewXSArgs"], sub {
-        my ($func_name,$pars) = @_;
-        my $shortpars = join ',',map $_->[0], @$pars;
-        my $longpars = join ",",map $_->[1]->get_decl($_->[0]), @$pars;
-        (PDL::PP::pp_line_numbers(__LINE__-1, "$func_name($shortpars);"),
-          "void $func_name($longpars)");
+   PDL::PP::Rule->new(["RunFuncCall","RunFuncHdr"],["RunFuncName","SignatureObj"], sub {
+        my ($func_name,$sig) = @_;
+        my $shortpars = join ',', $sig->alldecls(0, 0);
+        my $longpars = join ",", $sig->alldecls(1, 0);
+        (PDL::PP::pp_line_numbers(__LINE__-1, "PDL->barf_if_error($func_name($shortpars));"),
+          "pdl_error $func_name($longpars)");
       }),
 
    PDL::PP::Rule->new("NewXSMakeNow", ["SignatureObj"],
-      sub {
-        my($sig) = @_;
-        my $str = PDL::PP::pp_line_numbers(__LINE__-1, "\n");
-        for(@{ $sig->names }) { $str .= "$_ = PDL->make_now($_);\n"; }
-        $str;
-      }),
+      sub { join '', map PDL::PP::pp_line_numbers(__LINE__-1, "$_ = PDL->make_now($_);\n"), @{ $_[0]->names } }),
    PDL::PP::Rule->new("IgnoreTypesOf", ["FTypes","SignatureObj"], sub {
       my ($ftypes, $sig) = @_;
       my ($pnames, $pobjs) = ($sig->names_sorted, $sig->objs);
@@ -2072,38 +1995,40 @@ END
    PDL::PP::Rule->new("NewXSCoerceMustNS", "FTypes",
       sub {
         my($ftypes) = @_;
-        PDL::PP::pp_line_numbers(__LINE__, join '', map
-          "$_->datatype = $ftypes->{$_}; ",
-          keys %$ftypes);
+        join '', map
+          PDL::PP::pp_line_numbers(__LINE__, "$_->datatype = $ftypes->{$_};"),
+          sort keys %$ftypes;
       }),
    PDL::PP::Rule::Substitute::Usual->new("NewXSCoerceMustSubd", "NewXSCoerceMustNS"),
 
    PDL::PP::Rule->new("NewXSTypeCoerceNS", ["StructName"],
       sub {
-        PDL::PP::pp_line_numbers(__LINE__-1, "PDL->type_coerce($_[0]);");
+        PDL::PP::pp_line_numbers(__LINE__, <<EOF);
+PDL_RETERROR(PDL_err, PDL->type_coerce($_[0]));
+EOF
       }),
    PDL::PP::Rule::Substitute::Usual->new("NewXSTypeCoerceSubd", "NewXSTypeCoerceNS"),
 
    PDL::PP::Rule->new("NewXSSetTransPDLs", ["SignatureObj","StructName"], sub {
       my($sig,$trans) = @_;
       my $no=0;
-      PDL::PP::pp_line_numbers(__LINE__, join '',
-        map "$trans->pdls[".($no++)."] = $_;\n",
-        @{ $sig->names_sorted });
+      join '',
+        map PDL::PP::pp_line_numbers(__LINE__, "$trans->pdls[".($no++)."] = $_;\n"),
+        @{ $sig->names_sorted };
    }),
 
    PDL::PP::Rule->new("NewXSExtractTransPDLs", ["SignatureObj","StructName"], sub {
       my($sig,$trans) = @_;
       my $no=0;
-      PDL::PP::pp_line_numbers(__LINE__, join '',
-        map "$_ = $trans->pdls[".($no++)."];\n",
-        @{ $sig->names_sorted });
+      join '',
+        map PDL::PP::pp_line_numbers(__LINE__, "$_ = $trans->pdls[".($no++)."];\n"),
+        @{ $sig->names_sorted };
    }),
 
    PDL::PP::Rule->new("NewXSRunTrans", ["StructName"], sub {
       my($trans) = @_;
       PDL::PP::pp_line_numbers(__LINE__,
-      "PDL->make_trans_mutual($trans);\n");
+      "PDL_RETERROR(PDL_err, PDL->make_trans_mutual($trans));\n");
    }),
 
    PDL::PP::Rule->new(PDL::PP::Code::make_args("Code"),
@@ -2111,42 +2036,28 @@ END
    PDL::PP::Rule->new(PDL::PP::Code::make_args("BackCode"),
 		      sub { PDL::PP::Code->new(@_, undef, 1); }),
 
-# Compiled representations i.e. what the xsub function leaves
-# in the trans structure. By default, copies of the parameters
+# Compiled representations i.e. what the RunFunc function leaves
+# in the params structure. By default, copies of the parameters
 # but in many cases (e.g. slice) a benefit can be obtained
 # by parsing the string in that function.
-
-# If the user wishes to specify his own code and compiled representation,
-# The next two definitions allow this.
-# Because of substitutions that will be there,
-# makecompiledrepr et al are array refs, 0th element = string,
-# 1th element = hashref of translated names
-# This makes the objects: type + ...
-#
-   PDL::PP::Rule->new(["CompNames","CompObjs"], "Comp", \&OtherPars_nft),
-   PDL::PP::Rule->new("CompiledRepr", ["CompNames","CompObjs"], sub {NT2Decls__({ToPtrs=>1},@_)}),
-   PDL::PP::Rule::MakeComp->new("MakeCompiledReprNS", ["MakeComp","CompNames","CompObjs"],
+# If the user wishes to specify their own MakeComp code and Comp content,
+# The next definitions allow this.
+   PDL::PP::Rule->new("CompObj", ["BadFlag","Comp"],
+      sub { PDL::PP::Signature->new('', @_) }),
+   PDL::PP::Rule->new("CompObj", "SignatureObj", sub { @_ }), # provide default
+   PDL::PP::Rule->new("MakeCompOther", "SignatureObj", sub { $_[0]->getcopy }),
+   PDL::PP::Rule->new("MakeCompTotal", ["MakeCompOther", \"MakeComp"], sub { join "\n", grep $_, @_ }),
+   PDL::PP::Rule->new("CompStructOther", "SignatureObj", sub {$_[0]->getcomp}),
+   PDL::PP::Rule->new("CompStructComp", [qw(CompObj Comp)], sub {$_[0]->getcomp}),
+   PDL::PP::Rule->new("CompStruct", ["CompStructOther", \"CompStructComp"], sub { join "\n", grep $_, @_ }),
+   PDL::PP::Rule::MakeComp->new("MakeCompiledReprNS", ["MakeCompTotal","CompObj"],
 				"COMP"),
-
-   PDL::PP::Rule->new("CompFreeCode", ["CompNames","CompObjs"], sub {NT2Free_p(@_,"COMP")}),
-
-# This is the default
-#
-   PDL::PP::Rule->new("MakeCompiledReprNS",
-      ["OtherParNames","OtherParTypes","ParamStructName"],
-      sub {
-        my($onames,$otypes,$pname) = @_;
-        PDL::PP::pp_line_numbers(__LINE__,
-          join '', map $otypes->{$_}->get_copy($_,"$pname->$_"), @$onames
-        );
-      }),
-   PDL::PP::Rule->new("CompiledRepr",
-      ["OtherParNames","OtherParTypes"],
-      sub {NT2Decls__({},@_)}),
-   PDL::PP::Rule->new("CompFreeCode", ["OtherParNames","OtherParTypes"], sub {NT2Free_p(@_,"COMP")}),
+   PDL::PP::Rule->new("CompFreeCodeOther", "SignatureObj", sub {$_[0]->getfree("COMP")}),
+   PDL::PP::Rule->new("CompFreeCodeComp", [qw(CompObj Comp)], sub {$_[0]->getfree("COMP")}),
+   PDL::PP::Rule->new("CompFreeCode", ["CompFreeCodeOther", \"CompFreeCodeComp"], sub { join "\n", grep $_, @_ }),
 
    PDL::PP::Rule->new(["StructDecl","ParamStructType"],
-      ["CompiledRepr","Name"],
+      ["CompStruct","Name"],
       sub {
         my($comp,$name) = @_;
         return ('', '') if !$comp;
@@ -2155,15 +2066,17 @@ END
         $ptype);
       }),
 
+   PDL::PP::Rule::Substitute->new("MakeCompiledReprSubd", "MakeCompiledReprNS"),
+
    PDL::PP::Rule->new("DefaultRedoDims",
       ["StructName"],
-      sub { "PDL->redodims_default($_[0]);" }),
+      sub { "PDL_RETERROR(PDL_err, PDL->redodims_default($_[0]));" }),
 
    PDL::PP::Rule->new("DimsSetters",
       ["SignatureObj"],
-      sub { join "\n", map $_->get_initdim, $_[0]->dims_values }),
+      sub { join "\n", sort map $_->get_initdim, $_[0]->dims_values }),
 
-   PDL::PP::Rule->new("RedoDimsFuncName", [qw(Name _RedoDims _RedoDimsCode DimsSetters)],
+   PDL::PP::Rule->new("RedoDimsFuncName", ["Name", \"RedoDims", \"RedoDimsCode", "DimsSetters"],
       sub { (scalar grep $_ && /\S/, @_[1..$#_]) ? "pdl_$_[0]_redodims" : 'NULL'}),
    PDL::PP::Rule::Returns->new("RedoDimsCode", [],
 			       'Code that can be inserted to set the size of output ndarrays dynamically based on input ndarrays; is parsed',
@@ -2178,15 +2091,14 @@ END
 
    PDL::PP::Rule::Returns::EmptyString->new("Priv"),
 
-   PDL::PP::Rule->new(["PrivNames","PrivObjs"], "Priv", \&OtherPars_nft),
-   PDL::PP::Rule->new("NTPrivFreeCode", ["PrivNames","PrivObjs"], sub {NT2Free_p(@_,"PRIV")}),
-
-   PDL::PP::Rule::Substitute->new("MakeCompiledReprSubd", "MakeCompiledReprNS"),
+   PDL::PP::Rule->new("PrivObj", ["BadFlag","Priv"],
+      sub { PDL::PP::Signature->new('', @_) }),
+   PDL::PP::Rule->new("NTPrivFreeCode", "PrivObj", sub {$_[0]->getfree("PRIV")}),
 
    PDL::PP::Rule->new("FreeCodeNS",
       ["StructName","CompFreeCode","NTPrivFreeCode"],
       sub {
-	  (grep $_, @_[1..$#_]) ? PDL::PP::pp_line_numbers(__LINE__-1, "PDL_FREE_CODE($_[0], $_[1], $_[2])"): ''}),
+	  (grep $_, @_[1..$#_]) ? PDL::PP::pp_line_numbers(__LINE__-1, "PDL_FREE_CODE($_[0], destroy, $_[1], $_[2])"): ''}),
 
    PDL::PP::Rule::Substitute::Usual->new("FreeCodeSubd", "FreeCodeNS"),
 
@@ -2199,7 +2111,10 @@ END
    PDL::PP::Rule->new("NewXSFindBadStatusNS", ["StructName"],
       "Rule to find the bad value status of the input ndarrays",
       sub {
-        PDL::PP::pp_line_numbers(__LINE__-1, "char \$BADFLAGCACHE() = PDL->trans_badflag_from_inputs($_[0]);\n");
+        PDL::PP::pp_line_numbers(__LINE__, <<EOF);
+PDL_RETERROR(PDL_err, PDL->trans_check_pdls($_[0]));
+char \$BADFLAGCACHE() = PDL->trans_badflag_from_inputs($_[0]);
+EOF
       }),
 
    PDL::PP::Rule->new("NewXSCopyBadStatusNS",
@@ -2220,7 +2135,7 @@ END
       sub {
         my ( $sig ) = @_;
         return '' if @{$sig->names} == (my @outs = $sig->names_out); # no input pdls, no badflag copying needed
-        PDL::PP::pp_line_numbers(__LINE__-1, join '',
+        PDL::PP::pp_line_numbers(__LINE__, join '',
           "if (\$BADFLAGCACHE()) {\n",
           (map "  \$SETPDLSTATEBAD($_);\n", @outs),
           "}\n");
@@ -2259,46 +2174,28 @@ EOF
       "Generate C function with idiomatic arg list to maybe call from XS",
       sub {
         my ($xs_c_header, @bits) = @_;
-        PDL::PP::pp_line_numbers __LINE__-1, join '', "$xs_c_header {\n", @bits, "}\n";
+        my $opening = 'pdl_error PDL_err = {0, NULL, 0};';
+        my $closing = 'return PDL_err;';
+        PDL::PP::pp_line_numbers __LINE__-1, join '', "$xs_c_header {\n$opening\n", @bits, "$closing\n}\n";
       }),
 
- # Generates XS code with variable argument list.  If this rule succeeds, the next rule
- # will not be executed. D. Hunt 4/11/00
- #
+   # internal usage, not XS - NewXSCHdrs only set if GlobalNew
    PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
-      ["VarArgsXSHdr",@xscode_args_always,
-       "VarArgsXSReturn"
-      ],
+      ["NewXSHdr", "NewXSCHdrs", "RunFuncCall"],
+      "Non-varargs XS code when GlobalNew given",
+      sub {make_xs_code('CODE:',' XSRETURN(0);',@_)}),
+   # if PMCode supplied, no var-args stuff
+   PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
+      ["PMCode","NewXSHdr", \"NewXSCHdrs", "RunFuncCall"],
+      "Non-varargs XS code when PMCode given",
+      sub {make_xs_code('CODE:',' XSRETURN(0);',@_[1..$#_])}),
+   PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
+      [qw(VarArgsXSHdr), \"NewXSCHdrs", qw(RunFuncCall VarArgsXSReturn)],
       "Rule to print out XS code when variable argument list XS processing is enabled",
       sub {make_xs_code('','',@_)}),
 
- # This rule will fail if the preceding rule succeeds
- # D. Hunt 4/11/00
- #
-   PDL::PP::Rule->new(["NewXSCode","BootSetNewXS","NewXSInPrelude"],
-      ["NewXSHdr",@xscode_args_always],
-      "Rule to print out XS code when variable argument list XS processing is disabled",
-      sub {make_xs_code('CODE:',' XSRETURN(0);',@_)}),
-
-   PDL::PP::Rule::MakeComp->new("RedoDims-PostComp",
-      ["RedoDims", "PrivNames", "PrivObjs"], "PRIV"),
-
-   # The RedoDimsCodeNS rule takes in the RedoDims target
-   # directly as well as via RedoDims-PostComp for better error-reporting
-   PDL::PP::Rule->new("RedoDimsCodeNS",
-      ["RedoDims", "RedoDims-PostComp", "_DimsObj"],
-      sub {
-        my ($redodims, $result, $dimobjs) = @_;
-        $result->[1]{"SIZE"} = sub {
-          my ($dimname) = @_;
-          unless (defined $dimobjs->{$dimname}) {
-            eval { require PDL::IO::Dumper };
-            croak "can't get SIZE of undefined dimension (RedoDims=$redodims).\nredodims is $redodims\ndimobjs is ".PDL::IO::Dumper::sdump($dimobjs)."\n"
-          }
-          return $dimobjs->{$dimname}->get_size();
-        };
-        return $result;
-      }),
+   PDL::PP::Rule::MakeComp->new("RedoDimsCodeNS",
+      ["RedoDims", "PrivObj"], "PRIV"),
    PDL::PP::Rule::Substitute->new("RedoDimsCodeSubd", "RedoDimsCodeNS"),
    PDL::PP::Rule->new(make_vfn_args("RedoDims")),
 
@@ -2318,7 +2215,7 @@ EOF
    PDL::PP::Rule->new("FreeFuncName",
 		      ["FreeCodeSubd","Name"],
 		      sub {$_[0] ? "pdl_$_[1]_free" : 'NULL'}),
-   PDL::PP::Rule->new(make_vfn_args("Free")),
+   PDL::PP::Rule->new(make_vfn_args("Free", ", char destroy")),
 
    PDL::PP::Rule::Returns::Zero->new("NoPthread"), # assume we can pthread, unless indicated otherwise
    PDL::PP::Rule->new("VTableDef",
