@@ -57,11 +57,9 @@ void DFF(double* xval, double* vector){
   PUTBACK;
   px = PDL->SvPDLV(pxsv);
   
-  PDL->converttype( px, PDL_D );
-  PDL->children_changesoon(px,PDL_PARENTDIMSCHANGED|PDL_PARENTDATACHANGED);
+  PDL->barf_if_error(PDL->converttype( px, PDL_D ));
   PDL->setdims (px,pdims,ndims);
   px->state |= PDL_ALLOCATED | PDL_DONTTOUCHDATA;
-  PDL->changed(px,PDL_PARENTDIMSCHANGED|PDL_PARENTDATACHANGED,0);
 
   px->data = (void *) xval;
 
@@ -90,7 +88,7 @@ void DFF(double* xval, double* vector){
   pvectorsv = ST(0);
   pvector = PDL->SvPDLV(pvectorsv);
   
-  PDL->make_physical(pvector);
+  PDL->barf_if_error(PDL->make_physical(pvector));
   
   xpass  =  (double *) pvector->data;
   
@@ -139,44 +137,29 @@ print_state (size_t iter, gsl_multiroot_fsolver * s)
 
 int fsolver (double *xfree, int  nelem, double epsabs, int method) 
 {
-
   gsl_multiroot_fsolver_type *T;
   gsl_multiroot_fsolver *s;
-  
   int status;
   size_t i, iter = 0;
-  
   size_t n = nelem;
   double p[1] = { nelem };
   int iloop;
-
   //  struct func_params p = {1.0, 10.0};
-
   gsl_multiroot_function func = {&my_f, n,  p};
-  
   gsl_vector *x = gsl_vector_alloc (n);
-  
   for (iloop=0;iloop<nelem; iloop++) {
     //printf("in fsovler2D, C side, input is %g \n",xfree[iloop]);
     gsl_vector_set (x, iloop, xfree[iloop]);
   }
-  
-
   switch (method){
   case 0 : T = (gsl_multiroot_fsolver_type *) gsl_multiroot_fsolver_hybrids; break;
   case 1 : T = (gsl_multiroot_fsolver_type *) gsl_multiroot_fsolver_hybrid;  break;
   case 2 : T = (gsl_multiroot_fsolver_type *) gsl_multiroot_fsolver_dnewton; break;
   case 3 : T = (gsl_multiroot_fsolver_type *) gsl_multiroot_fsolver_broyden; break;
-  default: barf("Something is wrong: could not assing fsolver type...\n"); break;
+  default: return GSL_EINVAL; break;
   }
-  
-
   s = gsl_multiroot_fsolver_alloc (T, nelem);
-  
-
   gsl_multiroot_fsolver_set (s, &func, x);
-
- 
   do
     {
       iter++;
@@ -189,18 +172,12 @@ int fsolver (double *xfree, int  nelem, double epsabs, int method)
 	  gsl_multiroot_test_residual (s->f, epsabs);
     }
   while (status == GSL_CONTINUE && iter < 1000);
-  
   if (status) 
       warn ("Final status = %s\n", gsl_strerror (status));
-
   for (iloop=0;iloop<nelem; iloop++) {
     xfree[iloop] = gsl_vector_get (s->x, iloop);
   }
-  
   gsl_multiroot_fsolver_free (s);
   gsl_vector_free (x);
-
-
-  return 0;
-
+  return GSL_SUCCESS;
 }
